/*********************************************************************
  BBSDuino v1 :
  Console velo electrique (VAE) Bafang BBS0x Arduino sur Nano v3 + module I2C Oled 1306 (128*64), ligne jaune en haut, affichage en bleu
  Commodo 3 boutons récupéré  du display P850C (DCP14)
  Fils rallonge comodo, Higo B5 (femelle) :
  - noir (sur gnd) + brun = Bouton Centre
  - vert (sur gnd) + orange = Bouton Plus
  - vert (sur gnd) + blanc = Bouton Moins
  Connexions sur Nano en mode INPUT_PULLUP !!

  Option : Modusle I2c CAN 16 bits ADS1115 :  librairie voir https://learn.adafruit.com/adafruit-4-channel-adc-breakouts/
  Utilise pour le capteur de tension batterie (pont-div)  et module d'intensite pololu 30A (option), + divers possible

  // OLED 1306 lite !
  // Use smaller faster AvrI2c class in place of Wire // ... si choix !
  // Liste des fonts cf allFonts.h !
  // Librairie SSD1306Ascii.h voir lien https://forum.arduino.cc/index.php?topic=335594.0

  Alimentation : Module Recom RECOM R-78HB9.0-0.5 : 72V-9V 500mA : Voir Conrad.fr
  http://www.conrad.fr/ce/fr/product/676159/Convertisseur-CCCC-pour-circuits-imprimes-RECOM-R-78HB90-05-Nbr-de-sorties-1-x-72-VDC-9-VDC-05-A-45-W-1-pcs

  Mesures alim Nano + Oled uniquement :
  sortie Recom = 8.91V min sur 16V : 8.93V-8.94V sur 48V
  Regul 5V clone Nano sur USB = 4.62V
  Regul 5V clone Nano (source 49V ou 54V - Vin 8.93V / 8.94V)  = 4.7V à vide et 4.69V avec Oled !!
  Conso sur multimetre = 38V = ~7.4mA : 48V = ~6.3 mA : 54V = ~5.7mA

  Courant circulant sur le fil Pl du BBS (interupteur ou reset), ~20 mA sous 48V avec Nano + Oled + controleur sur ON

  NOTE IMPORTANTE :
  -------------------
  Pour l'instanr les capteurs ADS115 et capt. de courant Pololu ne sont pas installes (juste declares) !!
  La tension et les amperes sont reprise des retours serial du BBS0x !
  Voir partie "Constantes programme"

  © Jacques74 2016-2018
  Creative Commons, BY-NC-SA
  Attribution + Utilisation Non Commerciale + Partage dans les memes conditions
  http://creativecommons.fr/licences/


  IDE Arduino 1.8.3
  Forum Cyclurba
  sujet  : http://cyclurba.fr/forum/forum_discussion.php?from=91&discussionID=18542&messageID=568104&rubriqueID=102&pageprec=
  Sources et contributions : S.Penoff https://penoff.me/2016/01/13/e-bike-conversion-software/
  mais aussi SuperJc, Yamty35, Papat ...
Travail de Yamty35 : Analyse du protocole BBS02
https://github.com/weiss-jack/Bafang 

*********************************************************************/

#include "SSD1306Ascii.h" // librairie Oled 1306 lite
//#include "SSD1306AsciiAvrI2c.h"
#include "SSD1306AsciiWire.h"
#include <Streaming.h> //remplace print, ne consomme pas de ressources ! http://arduiniana.org/libraries/streaming/ 
#include <Wire.h>

//#include "Statistic.h" // Arduino Statistic lib http://playground.arduino.cc/Main/Statistics
#include <EEPROM.h>
#include <avr/wdt.h> // reset arduino, TEST !

#include <Adafruit_ADS1015.h> // CAN 16(15) bit
Adafruit_ADS1115 ads;  /* Use this for the 16-bit version */
// Adafruit_ADS1015 ads;     /* Use this for the 12-bit version */

// ********* OLED 1306 ********
// 0X3C+SA0 - 0x3C or 0x3D
#define OLED_I2C_ADDRESS 0x3C
// Define proper RST_PIN if required.
#define RST_PIN -1

// choix Wire ou AvrI2C Oled
//SSD1306AsciiAvrI2c oled;
SSD1306AsciiWire oled;


/********************** Arduino Nano v3 - E/S UTILISEES *************************/
// ------------- DIGITALES
// 0 RX (serial, BBS)
// 1 TX (serial, BBS)
const byte BTN_PLUS = 2; //D2
const byte BTN_MOINS = 3; //D3
const byte BTN_CENTRE = 4; //D4

// SoftwareSerial 10 (OpenLog ou Bluethoot)
// SoftwareSerial 11 (OpenLog ou Bluethoot)

// ------------ ANALOGIQUES
// les divers CAN sont remplaces par le module ADS1115 I2c 16 bit si option 
// A0 libre
// A1 libre
// A2 libre
// A3 libre
// A4 Oled + ADS1115 (I2C-SDA)
// A5 Oled + ADS1115 (I2C-SCL)
// A6 indisponible  sous module Oled (cicuit pastillé, voir images)
// A7 indisponible  sous module Oled (cicuit pastillé, voir images)



// ------------ Constantes programme
// BBS0x
const float CIRC_ROUE = 2.166; // circonference roue en m
const bool USE_CAPT_COURANT = 0; // capteur Pololu, 0=no, 1=yes (si 0 alors courant de GetBBS)
const bool USE_PONT_DIV = 0; // pont diviseur tension batterie, 0=no, 1=yes (si 0 alors tension de GetBBS)
// calibration
const float COEFF_PDIV = 11.32; // coefficent pont diviseur tension gloable (mesure 54.7=4.81V soit 11.372) + correction (option)
const float REF_AMP = 0.132; // capteur Pololu 30A, 133 mV par ampère + correction selon alim (option)
const float ZERO_AMP = 0.497; // Doc Pololu = 500 mV when the input current is zero : a mesurer et tester ! (option)
const float ADS_MULTIPLIER = 0.1875; // 1.875 mV par bit, cf ADS1115 Setup (option)
// Interval
const int LOG_INTERVAL = 500; // (1000) mills between entries (reduce to take more/faster data)
const int SYNC_INTERVAL = 500; // (1000) mills between calls to flush() - to write data to the card
const int DIVISEUR_INTERVAL = 7200; // (3600 à 1000ms et 7200 à 500 ms)


// ------------- Var globales ---------------
uint32_t syncTime = 0; // time of last sync()
float a_ref_calc = 0.00; // VCC ref calculée (cf fonction readVcc() )
byte numMenu = 0;
byte sMenu = 0;
float result_volt = 0.0; // tension batterie sur pont div et ADS1115
float result_amp = 0.0; // capteur ampère Pololu 30A et ADS1115
float result_watt = 0.0;
float old_Ah = 0.0; // calcul Ah temp
float new_Ah = 0.0; // calcul Ah
float OldWh = 0.0; // calcul Wh temp
float NewWh = 0.0; // calcul Wh
//--------- en développement...
float Vmin = 0.0;

// ----------- Var globales BBS
byte tabl_val[3] = {0x00, 0x00, 0x00}; // retours Get BBS
byte niveauBBS = 1;
byte PourcentBat = 0;
byte StatutError = 0;
unsigned int Rpm = 0;
float Kmh = 0.00;
float NewKm = 0.0; // calcul Km
float OldKm = 0.0; // calcul Km temp
float AmpBbs = 0.00;
float TensionBBS = 0.00; // en test !!

// Boutons
bool buttonStatePlus = 1; // 1 = état relaché
bool buttonStateMoins = 1;
bool buttonStateCenter = 1;

// -------- Structure permettant de stocker les variables en EEPROM 
struct Structure_EEP {
  float  valeur_1; // Ah
  float valeur_2; // Km
  float valeur_3; // Wh
}; // Ne pas oublier le point virgule !




//------------------------ SETUP -------------------
void setup() {
  Serial.begin(1200); // controleur BBS !
  Wire.begin();
  //Wire.begin(5, 4); // Wire.begin(SDA_pin, SCL_pin) Test ESP32 Oled, a creuser !
  Wire.setClock(400000L);
  pinMode(BTN_PLUS, INPUT_PULLUP); // Button Plus, resistance en interne !
  pinMode(BTN_MOINS, INPUT_PULLUP); // Button Moins, resistance en interne !
  pinMode(BTN_CENTRE, INPUT_PULLUP); // Button Centre, resistance en interne !

  pinMode(LED_BUILTIN, OUTPUT);

  //Oled
#if RST_PIN >= 0
  oled.begin(&Adafruit128x64, OLED_I2C_ADDRESS, RST_PIN);
#else // RST_PIN >= 0
  oled.begin(&Adafruit128x64, OLED_I2C_ADDRESS);
#endif // RST_PIN >= 0

  oled.clear();
  oled.setContrast(255); // 0 to 255

  // ADC 1115 16 bits
  // The ADC input range (or gain) can be changed via the following
  // functions, but be careful never to exceed VDD +0.3V max, or to
  // exceed the upper and lower limits if you adjust the input range!
  // Setting these values incorrectly may destroy your ADC!
  //                                                                ADS1015  ADS1115
  //                                                                -------  -------
  // ads.setGain(GAIN_TWOTHIRDS);  // 2/3x gain +/- 6.144V  1 bit = 3mV      0.1875mV (default)
  // ads.setGain(GAIN_ONE);        // 1x gain   +/- 4.096V  1 bit = 2mV      0.125mV
  // ads.setGain(GAIN_TWO);        // 2x gain   +/- 2.048V  1 bit = 1mV      0.0625mV
  // ads.setGain(GAIN_FOUR);       // 4x gain   +/- 1.024V  1 bit = 0.5mV    0.03125mV
  // ads.setGain(GAIN_EIGHT);      // 8x gain   +/- 0.512V  1 bit = 0.25mV   0.015625mV
  // ads.setGain(GAIN_SIXTEEN);    // 16x gain  +/- 0.256V  1 bit = 0.125mV  0.0078125mV

  ads.begin();
  delay(2000);

  // Code de réveil ou identifiant console ??
  // Serial.write(0x90);

  //Set BBS niv.1 par défaut
  niveauBBS = 1;
  Send_Niv(niveauBBS);
  GetError();
  numMenu = 1; // display principal par défaut
  sMenu = 0; // second display (3 choix max)

  // ---------------  Lit la structure en mémoire EEPROM et recupere les variables
  LoadEEPROM();

  // ----------- Init Vmin
  // a developper :  extraire valeur ou reprendre de l'eeprom
  // Vmin = result_volt;

  // accueil
  oled.home();
  // oled.setCursor(0, 0); // idem home
  // oled.set1X(); // pas necessaire ici
  //oled.setFont(Arial_bold_14);
  oled.setFont(TimesNewRoman16_bold);
  oled << "Hi Jack !" <<  endl; // ligne jaune
  oled << "-------------------" << endl;
  //oled << "Ah=" << _FLOAT(new_Ah, 1)  << "-  Wh=" << _FLOAT(NewWh, 0) << endl;
  //oled << "Km=" <<  _FLOAT(NewKm, 1) << endl;
  delay (1000);
  oled.clear();
}
// end setup 


//------------------------ LOOP -------------------
void loop() {
  int16_t adc0, adc2; // ADS1115

  // delay for the amount of time we want between readings
  delay((LOG_INTERVAL - 1) - (millis() % LOG_INTERVAL));
  // log milliseconds since starting
  uint32_t m = millis();
  //int MinutePassee = (m / 1000) / 60;

  //digitalWrite(LED_BUILTIN, HIGH);
  buttonStatePlus = digitalRead(BTN_PLUS);  // store current state btn PLUS 0=appui - 1=relaché
  buttonStateMoins = digitalRead(BTN_MOINS);  // store current state btn MOINS  0=appui - 1=relaché
  buttonStateCenter = digitalRead(BTN_CENTRE);  // store current state btn CENTRE  0=appui - 1=relaché

  //----------- Get BBS, cf serial_BBS
  ReinitTablVal();
  GetError();
  GetCharge();
  GetAmpBbs();
  GetTensionBBS(); // affichage test tension
  GetKmh(); // à placer en dernier si affichage d'essais RPM


  // ------------- VCC ref calculé, Conversion Float ! cf fonction readVcc(),
  // a_ref_calc = float(readVcc()) / 1000; // pas utilisé pour l'instant !


  // --------------------- Récupérer tension batterie (ADS1115)
  // result_volt : à placer dans une fonction
  // --------------------- Récupérer ampères (capteur Pololu 30A et ADS1115)
  // result_amp : à placer dans une fonction

  // -------- Selon capteurs installés :
  if (USE_CAPT_COURANT == 0) result_amp = AmpBbs;
  if (USE_PONT_DIV == 0) result_volt = TensionBBS;

  // ------------ Calcul et cumul des Ah - deux mesures par seconde (fréq. 500 ms) soit / 7200 pour une heure
  if (result_amp > 0.0) {
    new_Ah = result_amp / DIVISEUR_INTERVAL + old_Ah;
    old_Ah = new_Ah;
  }
  // --------- Calcul Watts et Wh
  if (result_amp > 0.0) {
    result_watt = result_volt * result_amp;
    NewWh = result_watt / DIVISEUR_INTERVAL + OldWh;
    OldWh = NewWh;
  }

  // ------------- Cumul des Km
  if (Kmh > 0.0) {
    NewKm = Kmh / DIVISEUR_INTERVAL + OldKm;
    OldKm = NewKm;
    // ou Mtot = (RPM * CIRC_ROUE) / 120; // (mètre par 500 ms)
    // Km = Mtot / 1000; // (Km par 500 ms)
  }

  // A développer : Calcul tension minimum ...
  if (result_volt < Vmin) Vmin = result_volt;

  // A développer : Calcul A maximum (ou W max) !
  //


  // -----------  Augmentation ou diminution niveaux + menus boutons
  if (buttonStatePlus == 0) {
    niveauBBS ++;
    if (niveauBBS > 9) niveauBBS = 0;
    if (niveauBBS < 0) niveauBBS = 0;
    numMenu = 1;
    sMenu = 0; // reinit choix affichage alternatif
    Send_Niv(niveauBBS); // send niv. BBS
    //oled.clear();
  }
  if (buttonStateMoins == 0) {
    niveauBBS --;
    if (niveauBBS > 9) niveauBBS = 0;
    if (niveauBBS < 0) niveauBBS = 0;
    numMenu = 1;
    sMenu = 0; // reinit choix affichage alternatif
    Send_Niv(niveauBBS); // send niv. BBS
    //oled.clear();
  }
  if (buttonStateCenter ==  0) {
    // 3 affichage alternatifs
    sMenu ++;
    if (sMenu > 4) sMenu = 1; // 4 = sortie du mode appui multiple !
    if (sMenu < 1) sMenu = 1;
    numMenu = 2;
    oled.clear();
  }
  // Si + et center appuyés en même temps
  // ----------- WRITE EEPROM
  // ---------   A lancer une seule fois avant de couper la batterie
  if ((buttonStatePlus == 0) && (buttonStateCenter == 0 )) {
    numMenu = 3; //  écriture en EEPROM et retour ok ...
    oled.clear();
  }
  // Si - et center appuyés en même temps
  // ----------- RESET EEPROM et compteur !
  if ((buttonStateMoins == 0) && (buttonStateCenter == 0 )) {
    numMenu = 4; // vider variables EEPROM (mise à 0)
    oled.clear();
  }

  // ---- REPROG -----
  // Si + et - appuyés en même temps
  if ((buttonStatePlus == 0) && (buttonStateMoins == 0 )) {
    // test RPM ou divers - Ou reprog Mode légal
    numMenu = 5;
    oled.clear();
  }
  // Si +, - et center appuyés en même temps (3 à la fois !!)
  if ((buttonStatePlus == 0) && (buttonStateMoins == 0 ) && (buttonStateCenter == 0 )) {
    // Reprog mode débridé
    numMenu = 6;
    oled.clear();
  }


  // ----------- Affichages et Menus
  switch (numMenu) {
    case 1:
      Affich_display_1();
      break;
    case 2:
      if (sMenu == 1) Affich_display_2a();
      if (sMenu == 2) Affich_display_2b();
      if (sMenu == 3) Affich_display_2c();
      if (sMenu == 4) Affich_display_1(); // sortie et retour affichage principal
      break;
    case 3:
      Affich_write_eeprom(); // écrit en EEPROM
      break;
    case 4:
      Affich_del_eeprom(); // vider EEPROM
      break;
    case 5:
      // Affich_RPM(tabl_val[0], tabl_val[1], tabl_val[2]);
      // Affich_TensionBBS(tabl_val[0], tabl_val[1]);
      Affich_Reprog_Legal();
      break;
    case 6:
      Affich_Reprog_Debrid();
      break;
    default:
      Affich_display_1();
      break;
  } //endswitch

  //digitalWrite(LED_BUILTIN, LOW);

  // -------- Interval envois et reception des données
  if ((millis() - syncTime) < SYNC_INTERVAL) return;
  syncTime = millis();

} // end loop 





